@@ -24,20 +24,13 @@ RSpec.describe Zstd do
       expect(compressed).to_not eq(user_json)
     end
 
-    it 'should support compression levels' do
-      compressed = Zstd.compress(user_json, 1)
-      expect(compressed).to be_a(String)
-      expect(compressed).to_not eq(user_json)
-    end
-
     it 'should support compression keyward args levels' do
       compressed = Zstd.compress(user_json, level: 1)
-      compressed_with_arg = Zstd.compress(user_json, 1)
       compressed_default = Zstd.compress(user_json)
       expect(compressed).to be_a(String)
       expect(compressed).to_not eq(user_json)
-      expect(compressed).to eq(compressed_with_arg)
-      expect(compressed_default.length).to be < compressed_with_arg.length
+      expect(compressed).to_not eq(compressed_default)
+      expect(compressed_default.length).to be < compressed.length
     end
 
     it 'should compress large bytes' do
@@ -125,7 +118,11 @@ RSpec.describe Zstd do
     describe 'Ractor' do
       it 'should be supported' do
         r = Ractor.new { Zstd.compress('abc') }
-        expect(Zstd.decompress(r.take)).to eq('abc')
+        # Ractor#take was replaced at Ruby 3.5.
+        # https://bugs.ruby-lang.org/issues/21262
+        result = r.respond_to?(:take) ? r.take : r.value
+
+        expect(Zstd.decompress(result)).to eq('abc')
       end
     end
   end
